/*********************************************************************

          cg.c: CyberGlove device driver

                                        Kenji with Tail

*********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>


#include "cg.h"


#define _DEBUG_ 1


#define NUM_COM	(4+1)	/* port is 4, 0-4 */

static	int		port_ref[NUM_COM] = {-1, -1, -1, -1, -1};

/* cg#1-4 get-light flag */
static	int	lit_flg[NUM_COM] = {NULL, OFF_CG, OFF_CG, OFF_CG, OFF_CG};

/* for shared memory */
#define SHMSZ	25		/* char 1byte * 18(22)sensor(degree) + light ... */
#define KEYCG	030700		/* 0x31c0 */
static key_t 	key[NUM_COM] = {KEYCG, KEYCG+1, KEYCG+2, KEYCG+3, KEYCG+4};
static int	shmid[NUM_COM];
static int	pid[NUM_COM];
static char	*shm[NUM_COM];

char 	 	_cg_server4shm_[256] = _CG_SERVER4SHM_;


int Init_cg(int cg_id, char *port, long int baud)
{
	int i, result = 0;
	char com[256];

	if(port_ref[cg_id] != -1){
		fprintf(stderr, "CyberGlove(%d) has been initialized\n", cg_id);
		return result;
	}

	if(_cg_server4shm_[0] != '\0'){
		sprintf(com, "%s %d %s %ld &", _cg_server4shm_, cg_id, port, baud);
		fprintf(stderr, "[%s]\n", com);
		system(com);
	} else {
		fprintf(stderr, "cg: running?: %s %d %s %ld\n",
			_CG_SERVER4SHM_, cg_id, port, baud);
	}

	if((shmid[cg_id] = shmget(key[cg_id], SHMSZ, IPC_CREAT | 0666)) < 0){
		perror("shmget(cg-p)");
		exit(1);
	}

	if((shm[cg_id] = shmat(shmid[cg_id], NULL, 0)) == (char *) -1){
		perror("shmat(cg-p)");
		exit(1);
	}
#if _DEBUG_
	fprintf(stderr, "memmory sheared in parent process(cg%d)\n", cg_id);
#endif
	*shm[cg_id] = 'e';
	for(i = 0; i < 60 && result != 1; i++){
		sleep(1);
		if(*shm[cg_id] == 'd'){
			result = 1;
			port_ref[cg_id] = 1;
		}
	}
#if _DEBUG_
	fprintf(stderr, "connected both process %d(cg%d)\n", result, cg_id);
#endif
	return result;
}


int Close_cg(int cg_id)
{
	if(port_ref[cg_id] != -1){
		*shm[cg_id] = 'c';
		*(shm[cg_id] + 1) = 'c';
		shmdt(shm[cg_id]);
		return 1;
	} else {
		fprintf(stderr, "cg(%d) has not been opened\n", cg_id);
		return 0;
	}
}


int ReInit_cg(int cg_id)
{
	*shm[cg_id] = 'c';
	*(shm[cg_id] + 1) = 'i';
	return 1;
}


int ReStart_cg(int cg_id)
{
	*shm[cg_id] = 'c';
	*(shm[cg_id] + 1) = 's';
	return 1;
}


int Request_cg(int cg_id, Angle_cg angle)
{
	int i;
	char c, *a, *shmp;

	while(*shm[cg_id] != 'd') sleep(1);

	shmp = shm[cg_id] + 3;
	a = (char *) angle;
	for(i = 0; i < 18; i++){
		*(a + i) = *(shmp + i);
	}

	return (int) *(shmp - 1);
}


int Light_cg(int cg_id, int req)
{
	lit_flg[cg_id] = req;

	while(*shm[cg_id] != 'd') sleep(1);

	switch(req){
	    case OFF_CG:
		*shm[cg_id] = 'c';
		*(shm[cg_id] + 1) = 'f';
		break;
	    case ON_CG:
		*shm[cg_id] = 'c';
		*(shm[cg_id] + 1) = 'n';
		break;
	    case LIT_CG:
		*shm[cg_id] = 'c';
		*(shm[cg_id] + 1) = 'l';
		break;
	    default:
		fprintf(stderr, "Light_cg: please give OFF_CG/ON_CG/LIT_CG.\n");
	}

	return req;
}


