/*********************************************************************

          ps.c: PolhemusSensor device driver

                                        Kenji with Tail

*********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#include "ps.h"


#define _DEBUG_ 1


static	int		port_ref = -1;

/* station#1-4 active flag */
static	int	stn_flg[5] = {NULL, ON_PS, OFF_PS, OFF_PS, OFF_PS};

/* for shared memory */
#define SHMSZ	100	/* float 4byte * 6degree * 4sensor + stylus ... */
#define KEYPS	1619	/* 0x0653 */
static key_t 	key = KEYPS;
static int	shmid;
static int	pid;
static char	*shm;

char 	 	_ps_server4shm_[256] = _PS_SERVER4SHM_;


int Init_ps(char *port, long int baud)
{
	int i, result = 0;
	char com[256];

	if(port_ref != -1){
		fprintf(stderr, "PolhemusSensor has been initialized\n");
		return result;
	}

	if(_ps_server4shm_[0] != '\0'){
		sprintf(com, "%s %s %ld &", _ps_server4shm_, port, baud);
		fprintf(stderr, "[%s]\n", com);
		system(com);
	} else {
		fprintf(stderr, "ps: running?: %s %s %ld\n",
			_PS_SERVER4SHM_, port, baud);
	}

	if((shmid = shmget(key, SHMSZ, IPC_CREAT | 0666)) < 0){
		perror("shmget(ps-p)");
		exit(1);
	}

	if((shm = shmat(shmid, NULL, 0)) == (char *) -1){
		perror("shmat(ps-p)");
		exit(1);
	}
#if _DEBUG_
	fprintf(stderr, "memmory sheared in parent process(ps)\n");
#endif
	*shm = 'e';
	for(i = 0; i < 60 && result != 1; i++){
		sleep(1);
		if(*shm == 'd'){
			result = 1;
			port_ref = 1;
		}
	}
#if _DEBUG_
	fprintf(stderr, "connected both process %d(ps)\n", result);
#endif
	return result;
}


int Close_ps(void)
{
	if(port_ref != -1){
		*shm = 'c';
		*(shm + 1) = 'c';
		shmdt(shm);
		return 1;
	} else {
		fprintf(stderr, "ps has not been opened\n");
		return 0;
	}
}


int Reset_ps(void)
{
	*shm = 'c';
	*(shm + 1) = 'r';
	return 1;
}


int SetStation_ps(int stn, int act)
{
	while(*shm != 'd') sleep(1);

	*shm = 'c';
	*(shm + 1) = 's';
	if(act == OFF_PS){
		*(shm + 2) = stn;
	}else /* if(act == ON_PS) */{
		*(shm + 2) = stn + 10;
	}
	stn_flg[stn] = act;
	return 1;
}


int PosRequest_ps(PosOri_ps p1, PosOri_ps p2, PosOri_ps p3, PosOri_ps p4)
{
	int i, j, k, btn = 0;
	char *f, *shmp;

	while(*shm != 'd') sleep(1);

	if(stn_flg[1] == ON_PS){
		f = (char *) p1;
		shmp = shm + 3;
		btn = (int) *(shmp++);
		for(i = 0; i < 24; i++){
			*(f++) = *(shmp++);
		}
	}
	if(stn_flg[2] == ON_PS){
		f = (char *) p2;
		shmp = shm + 28;
		for(i = 0; i < 24; i++){
			*(f++) = *(shmp++);
		}
	}
	if(stn_flg[3] == ON_PS){
		f = (char *) p3;
		shmp = shm + 52;
		for(i = 0; i < 24; i++){
			*(f++) = *(shmp++);
		}
	}
	if(stn_flg[4] == ON_PS){
		f = (char *) p4;
		shmp = shm + 76;
		for(i = 0; i < 24; i++){
			*(f++) = *(shmp++);
		}
	}
	return btn;
}


